/*
 * Copyright (c) 1996-2015 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include "acdemo.h"

#include <math.h>

#include "demoConst.h"
#include "demo_c_types.h"


/* --- Attribute SetSpeed ----------------------------------------------- */

/** Validation codel controlSpeed of attribute SetSpeed.
 *
 * Returns genom_ok.
 * Throws demo_INVALID_SPEED.
 */
genom_event
controlSpeed(demo_speed speedRef, genom_context self)
{
  /* insert your code */
  if(speedRef != demo_SLOW && speedRef != demo_FAST)
       return demo_INVALID_SPEED(self);
  return genom_ok;
}


/* --- Activity MoveDistance -------------------------------------------- */

/** Validation codel controlDistance of activity MoveDistance.
 *
 * Returns genom_ok.
 * Throws demo_TOO_FAR_AWAY.
 */
genom_event
controlDistance(double distRef, double position, genom_context self)
{
  double lposRef;		/* I am not sure I need this local copy... */
  demo_TOO_FAR_AWAY_detail tfa;

  lposRef = position + distRef;
  if (lposRef > DEMO_MACHINE_LENGTH/2 || lposRef < -DEMO_MACHINE_LENGTH/2) {
       tfa.overshoot = fabs(lposRef) - fabs(DEMO_MACHINE_LENGTH/2);
       return demo_TOO_FAR_AWAY(&tfa, self);
  }

  return genom_ok;
}


/* --- Activity GotoPosition -------------------------------------------- */

/** Validation codel controlPosition of activity GotoPosition.
 *
 * Returns genom_ok.
 * Throws demo_TOO_FAR_AWAY.
 */
genom_event
controlPosition(double posRef, genom_context self)
{
  demo_TOO_FAR_AWAY_detail tfa;

  if (posRef > DEMO_MACHINE_LENGTH/2 || posRef < -DEMO_MACHINE_LENGTH/2) {
       tfa.overshoot = fabs(posRef) - fabs(DEMO_MACHINE_LENGTH/2);
       return demo_TOO_FAR_AWAY(&tfa, self);
  }
  return genom_ok;
}


/* --- Activity Monitor ------------------------------------------------- */

/** Validation codel controlPosition of activity Monitor.
 *
 * Returns genom_ok.
 * Throws demo_TOO_FAR_AWAY.
 */
/* already defined in service GotoPosition validation */
